"""
To see which endpoints are available, you can read the API docs at
https://bybit-exchange.github.io/docs/inverse/#t-introduction

Some methods will have required parameters, while others may be optional.
The arguments in pybit methods match those provided in the Bybit API
documentation.

The following functions are available:

exit()

Public Methods:
------------------------
get_orderbook()
get_klines()
get_tickers()
get_trading_records()
get_symbols()

Private Methods:
(requires authentication)
------------------------
place_active_order()
get_active_order()
cancel_active_order()
cancel_all_active_orders()
replace_active_order()
query_active_order()

place_conditional_order()
get_conditional_order()
cancel_conditional_order()
cancel_all_conditional_orders()
replace_conditional_order()
query_conditional_order()

user_leverage()
change_user_leverage()

my_position()
change_margin()
set_trading_stop()

get_risk_limit()
set_risk_limit()

get_last_funding_rate()
my_last_funding_fee()
predicted_funding_rate()

api_key_info()

get_wallet_balance()
wallet_fund_records()
withdraw_records()
user_trade_records()

server_time()
announcement()

Custom Methods:
(requires authentication)
------------------------
close_position()

"""

# Import pybit and define the HTTP object.
from pybit import HTTP

"""
You can create an authenticated or unauthenticated HTTP session. 
You can skip authentication by not passing any value for api_key
and api_secret.
"""

# Unauthenticated
session_unauth = HTTP(endpoint='https://api.bybit.com')

# Authenticated
session_auth = HTTP(
    endpoint='https://api.bybit.com',
    api_key='...',
    api_secret='...'
)

# Lets get market information about EOSUSD. Note that 'symbol' is
# a required parameter as per the Bybit API documentation.
session_unauth.latest_information_for_symbol(symbol='EOSUSD')

# We can fetch our wallet balance using an auth'd session.
session_auth.get_wallet_balance(coin='BTC')



class FailedRequestError(Exception):
    """
    Exception raised for failed requests.

    Attributes:
        message -- Explanation of the error.
        status_code -- The code number returned.
    """
    def __init__(self, message, status_code, time):
        self.message = message
        self.status_code = status_code,
        self.time = time
        super().__init__((
            f'{message} (ErrCode: {status_code}) (ErrTime: {time})'
        ))


class InvalidRequestError(Exception):
    """
    Exception raised for returned Bybit errors.

    Attributes:
        message -- Explanation of the error.
        status_code -- The code number returned.
    """
    def __init__(self, message, status_code, time):
        self.message = message
        self.status_code = status_code,
        self.time = time
        super().__init__((
            f'{message} (ErrCode: {status_code}) (ErrTime: {time} UTC)'
        ))



"""
To see which endpoints and topics are available, check the Bybit API 
documentation: https://bybit-exchange.github.io/docs/inverse/#t-websocket

Inverse Perpetual endpoints:
wss://stream-testnet.bybit.com/realtime
wss://stream.bybit.com/realtime

USDT Perpetual endpoints:
wss://stream-testnet.bybit.com/realtime_public
wss://stream-testnet.bybit.com/realtime_private
wss://stream.bybit.com/realtime_public
wss://stream.bybit.com/realtime_private

Public Topics:
orderBookL2_25
orderBookL2-200
trade
insurance
instrument_info
klineV2

Private Topics:
position
execution
order
stop_order
wallet
"""

# Import the WebSocket object from pybit.
from pybit import WebSocket

"""
We can also import the HTTP object at the same time using:

from pybit import HTTP, WebSocket

Additionally, we can simply import all of pybit and use each
object selectively.

import pybit
client = pybit.HTTP(...)
ws = pybit.WebSocket(...)
"""

# Define your endpoint URL and subscriptions.
endpoint = 'wss://stream.bybit.com/realtime'
subs = [
    'orderBookL2_25.BTCUSD',
    'instrument_info.100ms.BTCUSD',
    'instrument_info.100ms.ETHUSD'
]

# Connect without authentication!
ws_unauth = WebSocket(endpoint, subscriptions=subs)

# Connect with authentication!
ws_auth = WebSocket(
    endpoint,
    subscriptions=['position'],
    api_key='...',
    api_secret='...'
)

# Let's fetch the orderbook for BTCUSD.
print(
    ws_unauth.fetch('orderBookL2_25.BTCUSD')
)

# We can also create a dict containing multiple results.
print(
    {i: ws_unauth.fetch(i) for i in subs}
)

# Check on your position. Note that no position data is received until a
# change in your position occurs (initially, there will be no data).
print(
    ws_auth.fetch('position')
)
